# main code to compile 1-line text file for pdic conversion v0.4
#   uses index text file (dic-idx.txt) and dictionary text file (dic.txt)
#   (eg. outputs from DDWIN 2.30 epwing reader)
#   perl script encoded in shift-jis for parsing under jperl
#
#   (c) under GPLv3 diamondsky.org 2008

# USAGE:
# jperl pdicconv.pl -t <dic type> -o <output file> -delgaiji (optional)

# Change the default dictionary type (used when no parameter given)
my $dictype = "shinwaeidai-kana";	
					# change shinwaeidai-kana to one of the following types:
					#   shinwaeidai-kana	(j-e kenkyusha kana index)
					#   shinwaeidai-kanji	(j-e kenkyusha kanji index)
					#   shineiwadai		(e-j kenkyusha)				
					#   examples		(edict examples dic)
					#   kanjidic-fpw		(edict kanjidic freepwing)
					#   nofilter			(attempt generic conversion)
my $delgaiji = 0;		# set to 1 to delete all [] epwing gaiji tags
					# User customisation values
my $maxlength = 8000;	# set maximum part length in bytes. higher values may
					#  result in skipped entries during pdic conversion.
					#  values > ~13500 will cause problems in old versions
my $outputname = "pdic.txt"; # output filename

if ($#ARGV >= 0) {
  for ($argnum=0; $argnum<=$#ARGV; $argnum++) {
	if ($ARGV[$argnum] eq "-delgaiji") {
	   $delgaiji = 1;
	}
	elsif ($ARGV[$argnum] eq "-o") {
	   $outputname = $ARGV[$argnum + 1];
	   $argnum++
	} 
	elsif ($ARGV[$argnum] eq "-t") {
	   $dictype = $ARGV[$argnum + 1];
	   $argnum++
	}
	else {
	   print "\nCould not process parameters.\n";
	   print "Usage:\n";
	   print "jperl pdicconv.pl -t <dic type> -o <output file> -delgaiji (optional)\n";
	   exit 0;
	}
  }  
}

if ($dictype =~ /examples/) {
  $example = 1;
} else {
  $example = 0;		# change to 1 to create example entries as default
}
if ($dictype eq "examples" || $dictype eq "kanjidic-fpw") {
  $deleteindent=1;
} else {
  $deleteindent=0;		# set to 1 to delete indent from contents
}
if ($dictype =~/shinwaeidai/ || $dictype eq "kanjidic-fpw") {
  $yomikakiconvert=1;	
} else {
  $yomikakiconvert=0;	# set to 1 to find kana/kanji yomi/kaki for entry
}
if ($dictype eq "kanjidic-fpw") {
  $lookupfirstword=1;
} else {
  $lookupfirstword=0;	# set to 1 to use first word only as lookup output format
}
if ($dictype eq "examples") {
  $maxparts = 2;
} else {
  $maxparts = 999;		# due to current pdic formatting constraints, entries are
}					#   broken up into #x parts - you can also limit the entry
					#   size by changing this value (in multiples of $maxlength)

# Rest are for program initialisation
my $lookuplimit=60;		# index word length - perl causes error with long searches
my $lookie="";
my $yomikaki="";
my $partno="1";
my $skipcontent=1;		# flag skip content from start until lookup found
my $curentry="";
my $linelength=0;
my $word=-1;				# first lookup called in loop
my $line=1;

#open dic, "<:encoding(UTF-8)",     "dic.txt";
#open idx, "<:encoding(UTF-8)",     "dic-idx.txt";
open (dic, "dic.txt");
open (idx, "dic-idx.txt");
open (pdic, ">$outputname");

sub lookupfilter
# filter, delete kanji, romaji
{
  $out = shift;
  if ($out !~ /\S/) {
	return $out;
  }
  #print "$word: $out -> ";
  if ($dictype eq "shineiwadai")
  {
	#$out =~ s/(^.*)\n/$1/;
	#$out=~ s/(^.*?)\/.*/$1/;		# too weak for  DDWIN 2.6 eiwa
	$out=~ s/(^.[\w|,|\-|\|'|"|(|)|*|%|@|!|\s]*).*\n/$1/;	
  }
  elsif ($dictype ne "exactlookup")
  {
	$out =~ s/(^.*)\n/$1/;
  }
  if (length($out) >= $lookuplimit) {
	$out = substr($out, 0, $lookuplimit);
  }
  #print "$out \n";
  $out = quotemeta($out);			# quote shift-jis characters etc.
  return $out;
}

sub lookupformat
# format lookup output
{
  $out = shift;
  $out =~ s/[\\]//g;				# filter quotes etc
  
  if ($dictype eq "shineiwadai") {
     $out =~ s/[\]//g;				# filter spacers etc
  }
  elsif ($dictype eq "shinwaeidai-kana") {
	$out =~ s/(^.+\S)\s*\y.*/$1/;		# delete kanji
	$out =~ s/(^.+\S)\s*\(.*/$1/;		# delete romaji
  }
  elsif ($dictype eq "shinwaeidai-kanji") {
	if ( $out !~ /\y/ || $out =~ /\y\w/ ) {	# ignore english entries
	   #print "Skip: $word\n";
	   return "";					# no kanji -> skip entry
	}
	$out =~ s/^.*\y(.+)\z.*/$1/;		# just kanji
  }
  elsif ($dictype eq "examples") {
	$out =~ s/(^.+\S)\s*\[.*/$1/;		# delete kana
  }
  elsif ($lookupfirstword) {
	$out =~ s/(^\S+)\s.*$/$1/;		# use first word
  }
  if ($delgaiji) {					# delete gaiji
	$out =~ s/\[[z|h|Z|H]\w{4}\]/\E/g;
  }
  $out =~ s/(^.*\S)\s+$/$1/;			# delete ending white space
  unless ($dictype eq "kanjidic-fpw") {
	$out =~ s/(^.*\S)(\d+)$/$1 $2/;	# add space before number index
  }
  $out =~ s/(^.*)\n/$1/; 			# delete line break
  #print "Output: $out;\n ";
  return $out;
}

sub contentformat
# format entry content
{
  if ($deleteindent) {
	s/\s*(\S.*$)/$1/;			# delete indent
  }
  if ($delgaiji) {				# delete gaiji
	s/\[[z|h|Z|H]\w{4}\]/\E/g;
  }
}

sub isbadlookup
# skip lookup if blank or sub entry
{
  $look = shift;
  if ($look !~ /\S/) {
	return 1;
  }
  elsif ($dictype =~ /waei/ && $look =~ /^[\|\]/) {
	return 1;
  }
  return 0;
}

sub isbadcontent
# skip content if blank or flagged
{
  if (!/\S/) {
	return 1;
  }
  elsif ($dictype eq "kanjidic-fpw" && /^   /) {
	return 1;
  }
  return 0;
}

sub skipentry
# skip entry if returned true
{
  $look = shift;
  if ($dictype eq "kanjidic-fpw") {
#	if ($word > 6355) {						# keep only kanji
	if ($word > 6355 && $look !~ /^[a-z]+\d/) {	# keep kanji and pinyin
	   return 1;
	}
  }
  return 0;
}

sub getyomikaki
# get yomi, kakikata or entry info
{
  $out = shift;
  $out =~ s/[\\]//g;			# filter quotes etc
  if ($dictype eq "shinwaeidai-kana") {
  	if ( $out !~ /\y/ || $out =~ /\y\w/ ) {	# ignore english entries
	   #print "No kanji: $word\n";
	   return "";				# no kanji
	}
	$out =~ s/^.*\y(.+)\z.*/$1/;	# get kanji
  } 
  elsif ($dictype eq "shinwaeidai-kanji")
  {
	if ( $out !~ m/\y/ || $out =~ m/\y\w/ ) {	# ignore english entries
	   #print "No kana, skipping: $word\n";
	   return "";				# no kanji -> skip entry
	}
	$out =~ s/(^.+\S)\s*\y.*/$1/;	# delete kanji
	$out =~ s/(^.+\S)\s*\(.*/$1/;	# delete romaji
	$out =~ s/(^.*)\d+$/$1/;		# delete number index
  }
  elsif ($dictype eq "kanjidic-fpw") {
	if ( $out =~ /\s/ ) {
        $out =~ s/^\S+\s+(.*$)/$1/;	# get character info
	} else {
	   return "";
	}
  }
  $out =~ s/(^.*\S)\s+$/$1/;		# delete ending white space
  $out =~ s/(^.*)\n/$1/;			# delete line break
  #print "yomi/kaki: $out;\n ";
  return $out;
}

sub addlookup
# add new lookup entry
{
  ($look, $yomi) = @_;
  if ($word==0) { 
     s/(^.*)\n/$look \/\/\/ /; 	# using argument as replacement lookup
  } 
  else {
     s/(^.*)\n/\n$look \/\/\/ /;
  }
  if ($yomi =~ /\S/) {			# use 2nd argument for yomi/kaki
     #s/(^.*)/$1$yomikaki/;
	$_ = $_ . $yomi;
	$line=2;
  }
  $addedentries++;
}

sub addline
# add line of content
{
  if ($line == 1) {
	if ($example) {
	   s/(^.*)\n/\ \/ $1/;
	}
	else {
	   s/(^.*)\n/$1/;
	}
  } else {
	s/(^.*)\n/ \\ $1/;
  } 
  $line++;
}

sub addpart
# add new entry part (to deal with pdic data format problems)
{
  $part = shift;
  if ($example) {
	s/(^.*)\n*/\n$part \/\/\/ \ \/ $1/;
  }
  else {
	s/(^.*)\n*/\n$part \/\/\/ $1/;
  }
  $line= 2;
}

my @lookup = <idx>;
print "\nFormatting contents to $dictype 1-line pdic text file.\n";
print "Total words:";
print $#lookup + 1;
print "\n";
print "Progress:\n";

while (<dic>)
{
  if (isbadlookup($lookie)) {
	while (isbadlookup($lookie) && $word < $#lookup) {
	   #print "$word: Blank/skipped index - $lookie\n";
	   $word++;
	   $lookie = @lookup[$word];
	   $lookie = lookupfilter($lookie);
	}
  }
  
  $skipcontent = isbadcontent;
  
  if (/^$lookie/ && $word <= $#lookup) {
	if (!skipentry($lookie)) {
        if ($yomikakiconvert) {		# get yomikaki for j-e dictionariesa
		$yomikaki = getyomikaki($lookie);
	   }
	   $lookie = lookupformat($lookie);
	   if ($lookie !~ /\S/) {
		#print "Skip: $word\n";
		$skipcontent = 1;
	   } else {
		$line=1;
		addlookup($lookie, $yomikaki);
		$skipcontent = 0;
	   }
	}
	if ($word / 500 == int($word / 500)) {print "$word: $lookie ;  ";}
	$curentry=$lookie;	
	$word++;
	if ($word > $#lookup) {
	   print "\nFinal lookup is $word: $curentry\n";
	} 
	else {
	   $lookie = @lookup[$word];
	   $lookie = lookupfilter($lookie);	#filter from lookup in eiwa etc
	}
	$linelength=0;
	$partno="1";
  } 
  elsif (!$skipcontent) {
	contentformat;
	if (!/\S/) {
	   s/^.*\n//;
	} 
	elsif ($linelength + length >= $maxlength) {	#orig pdic entry size limit
	   if ($partno != $maxparts) {
	     $partno++;
	     $lookup_part = $curentry . " \(\#$partno\)";
		#print "$word: $lookup_part;  ";
		addpart($lookup_part);
	   }
	   else {
		$skipcontent=1;
	   }
	   $linelength = 0;
	}
	else {
	   addline;
	}
  }
  
  if (!$skipcontent) {
	print pdic $_;			# write to output file
	$linelength += length;
  }
}

close dic;
close idx;
close pdic;
print "Number of entries added: $addedentries\n";
exit 1;